@@ -8,6 +8,7 @@ use GooCanvas2;
 use Glib 1.220 qw(TRUE FALSE);    # To get TRUE and FALSE
 use HTML::Entities;
 use Carp;
+use POSIX qw/ceil/;
 use Readonly;
 Readonly my $_100_PERCENT       => 100;
 Readonly my $_360_DEGREES       => 360;
@@ -17,7 +18,7 @@ my $EMPTY = q{};
 my $device;
 my %old_idles;
 
-our $VERSION = '2.6.5';
+our $VERSION = '2.7.0';
 
 use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
     'zoom-changed' => {
@@ -125,16 +126,14 @@ sub set_text {
         }
     }
     $root = GooCanvas2::CanvasGroup->new;
-    $self->set_root_item($root);
-    if ( not defined $page->{w} ) {
+    my ( $width, $height ) = $page->get_size;
+    my ( $xres,  $yres )   = $page->get_resolution;
 
-        # quotes required to prevent File::Temp object being clobbered
-        my $pixbuf = Gtk3::Gdk::Pixbuf->new_from_file("$page->{filename}");
-        $page->{w} = $pixbuf->get_width;
-        $page->{h} = $pixbuf->get_height;
-    }
-    $self->{pixbuf_size} = { width => $page->{w}, height => $page->{h} };
-    $self->set_bounds( 0, 0, $page->{w}, $page->{h} );
+    # Commenting out the scaling factor, as it segfaults in GooCanvas2 v0.06
+    # $root->set_scale($yres / $xres, 1);
+    $self->set_root_item($root);
+    $self->{pixbuf_size} = { width => $width, height => $height };
+    $self->set_bounds( 0, 0, $width, $height );
 
     my $style        = $self->get_style_context;
     my $color_string = $style->get_color('normal')->to_string;
@@ -144,14 +143,16 @@ sub set_text {
     }
 
     # Attach the text to the canvas
+    $self->{confidence_list} = [];
     for my $box ( @{ $page->boxes } ) {
         my %options = (
-            root           => $root,
-            box            => $box,
-            transformation => [ 0, 0, 0 ],
-            edit_callback  => $edit_callback,
-            text_color     => $color_hex,
-            idle           => $idle,
+            root            => $root,
+            box             => $box,
+            transformation  => [ 0, 0, 0 ],
+            edit_callback   => $edit_callback,
+            text_color      => $color_hex,
+            idle            => $idle,
+            confidence_list => $self->{confidence_list},
         );
         if ($idle) {
             $old_idles{$box} = Glib::Idle->add(
@@ -169,6 +170,60 @@ sub set_text {
     return;
 }
 
+sub get_first_text {
+    my ($self) = @_;
+    $self->{confidence_index} = 0;
+    return $self->get_text_by_index;
+}
+
+sub get_previous_text {
+    my ($self) = @_;
+    if ( $self->{confidence_index} > 0 ) {
+        $self->{confidence_index} -= 1;
+    }
+    return $self->get_text_by_index;
+}
+
+sub get_next_text {
+    my ($self) = @_;
+    if ( $self->{confidence_index} < $#{ $self->{confidence_list} } ) {
+        $self->{confidence_index} += 1;
+    }
+    return $self->get_text_by_index;
+}
+
+sub get_last_text {
+    my ($self) = @_;
+    $self->{confidence_index} = $#{ $self->{confidence_list} };
+    return $self->get_text_by_index;
+}
+
+sub get_text_by_index {
+    my ($self) = @_;
+    return $self->{confidence_list}[ $self->{confidence_index} ][0];
+}
+
+# FIXME: bbox should have its own class, the bbox knows its confidence, and we
+# don't need to pass it here.
+
+sub update_index_by_text {
+    my ( $self, $text, $confidence ) = @_;
+
+    # There may be multiple boxes with the same confidence, so use a binary
+    # search to find the next smallest confidence, and then a linear search to
+    # find the box
+    my $l =
+      confidence_binary_search( $self->{confidence_list}, $confidence - 1 );
+    for my $i ( $l .. $#{ $self->{confidence_list} } ) {
+        if ( $self->{confidence_list}->[$i][0] == $text ) {
+            $self->{confidence_index} = $i;
+            return $i;
+        }
+    }
+    delete $self->{confidence_index};
+    return;
+}
+
 sub get_pixbuf_size {
     my ($self) = @_;
     return $self->{pixbuf_size};
@@ -244,7 +299,7 @@ sub _boxed_text {
     # and receives any mouse clicks
     my $confidence =
       defined $box->{confidence} ? $box->{confidence} : $_100_PERCENT;
-    $confidence = $confidence > 64    ## no critic (ProhibitMagicNumbers)
+    $confidence = $confidence > 64           ## no critic (ProhibitMagicNumbers)
       ? 2 * int( ( $confidence - 65 ) / 12 ) ## no critic (ProhibitMagicNumbers)
       + 10                                   ## no critic (ProhibitMagicNumbers)
       : 0;
@@ -292,6 +347,9 @@ sub _boxed_text {
             'font'       => 'Sans',
             'fill-color' => $text_color,
         );
+        add_box_to_confidence_list( $options{confidence_list},
+            $text, $box->{confidence} );
+
         my $angle  = -( $textangle + $rotation ) % $_360_DEGREES;
         my $bounds = $text->get_bounds;
         if ( ( $bounds->x2 - $bounds->x1 ) == 0 ) {
@@ -316,27 +374,28 @@ sub _boxed_text {
                     my ( $widget, $target, $event ) = @_;
                     if ( $event->button == 1 ) {
                         $root->get_parent->{dragging} = FALSE;
-                        $edit_callback->( $widget, $target, $event );
+                        $edit_callback->( $widget, $target, $event, $g );
                     }
                 }
             );
         }
     }
     if ( $box->{contents} ) {
-        for my $box ( @{ $box->{contents} } ) {
+        for my $child ( @{ $box->{contents} } ) {
             my %noptions = (
-                root           => $g,
-                box            => $box,
-                transformation => [ $textangle + $rotation, $x1, $y1 ],
-                edit_callback  => $edit_callback,
-                text_color     => $text_color,
-                idle           => $idle,
+                root            => $g,
+                box             => $child,
+                transformation  => [ $textangle + $rotation, $x1, $y1 ],
+                edit_callback   => $edit_callback,
+                text_color      => $text_color,
+                idle            => $idle,
+                confidence_list => $options{confidence_list},
             );
             if ($idle) {
-                $old_idles{$box} = Glib::Idle->add(
+                $old_idles{$child} = Glib::Idle->add(
                     sub {
                         _boxed_text(%noptions);
-                        delete $old_idles{$box};
+                        delete $old_idles{$child};
                         return Glib::SOURCE_REMOVE;
                     }
                 );
@@ -372,10 +431,63 @@ sub _boxed_text {
     return;
 }
 
+# Return index of confidence using binary search
+# https://en.wikipedia.org/wiki/Binary_search_algorithm#Alternative_procedure
+
+sub confidence_binary_search {
+    my ( $confidence_list, $confidence ) = @_;
+    my $l = 0;
+    my $r = $#{$confidence_list};
+    while ( $l != $r ) {
+        my $m = ceil( ( $l + $r ) / 2 );
+        if ( $confidence_list->[$m][1] > $confidence ) {
+            $r = $m - 1;
+        }
+        else {
+            $l = $m;
+        }
+    }
+    if ( $confidence_list->[$l][1] < $confidence ) {
+        $l += 1;
+    }
+    return $l;
+}
+
+# insert into list sorted by confidence level using a binary search
+
+sub add_box_to_confidence_list {
+    my ( $confidence_list, $text, $confidence ) = @_;
+    if ( not @{$confidence_list} ) {
+        push @{$confidence_list}, [ $text, $confidence ];
+        return;
+    }
+    my $i = confidence_binary_search( $confidence_list, $confidence );
+    splice @{$confidence_list}, $i, 0, [ $text, $confidence ];
+    return;
+}
+
+sub delete_box {    #FIXME: bbox should have its own class
+    my ( $self, $widget ) = @_;
+    my $g      = $widget->get_property('parent');
+    my $parent = $g->get_property('parent');
+    for my $i ( 0 .. $parent->get_n_children - 1 ) {
+        my $group = $parent->get_child($i);
+        if ( $group eq $g ) {
+            $parent->remove_child($i);
+            last;
+        }
+    }
+    splice @{ $self->{confidence_list} }, $self->{confidence_index}, 1;
+    if ( $self->{confidence_index} > $#{ $self->{confidence_list} } ) {
+        $self->{confidence_index} = $#{ $self->{confidence_list} };
+    }
+    return;
+}
+
 # Set the text in the given widget
 
-sub set_box_text {
-    my ( $self, $widget, $text ) = @_;
+sub update_box {    #FIXME: bbox should have its own class
+    my ( $self, $widget, $text, $selection ) = @_;
 
     # per above: group = text's parent, group's 1st child = rect
     my $g    = $widget->get_property('parent');
@@ -390,22 +502,27 @@ sub set_box_text {
 
         # re-adjust text size & position
         if ( $g->{type} ne 'page' ) {
-            my ( $x1, $y1, $x2, $y2 ) = @{ $g->{bbox} };
-            my $x_size = abs $x2 - $x1;
-            my $y_size = abs $y2 - $y1;
+            my ( $x1, $y1 ) = ( $selection->{x}, $selection->{y} );
+            my ( $x2, $y2 ) =
+              ( $x1 + $selection->{width}, $y1 + $selection->{height} );
+            $g->{bbox} = [ $x1, $y1, $x2, $y2 ];
             $widget->set_simple_transform( 0, 0, 1, 0 );
             my $bounds = $widget->get_bounds;
-            my $angle = $g->{_angle} || 0;
+            my $angle  = $g->{_angle} || 0;
             my $scale =
-              ( $angle ? $y_size : $x_size ) / ( $bounds->x2 - $bounds->x1 );
+              ( $angle ? $selection->{height} : $selection->{width} ) /
+              ( $bounds->x2 - $bounds->x1 );
 
             _transform_text( $g, $widget, $scale, $angle );
         }
+
+        # move position in confidence_list
+        splice @{ $self->{confidence_list} }, $self->{confidence_index}, 1;
+        add_box_to_confidence_list( $self->{confidence_list},
+            $widget, $g->{confidence} );
     }
     else {
-        delete $g->{text};
-        $g->remove_child(0);
-        $g->remove_child(1);
+        $self->delete_box($widget);
     }
     return;
 }
@@ -436,7 +553,7 @@ sub hocr {
     my ($self) = @_;
     if ( not defined $self->get_pixbuf_size ) { return }
     my ( $x, $y, $w, $h ) = $self->get_bounds;
-    my $root = $self->get_root_item;
+    my $root   = $self->get_root_item;
     my $string = _group2hocr( $root, 2 );
     return <<"EOS";
 <?xml version="1.0" encoding="UTF-8"?>
@@ -572,7 +689,7 @@ sub _button_pressed {
         # only the value since the last event is required.
         my ( $screen, $x, $y ) = $device->get_position;
         $self->{drag_start} = { x => $x, y => $y };
-        $self->{dragging} = TRUE;
+        $self->{dragging}   = TRUE;
 
         #    $self->update_cursor( $event->x, $event->y );
     }
