@@ -296,7 +296,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.6.5';
+our $VERSION = '2.7.0';
 
 my ( $d_sane, $logger );
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
@@ -1656,6 +1656,7 @@ sub set_paper {
     my ( $self, $paper ) = @_;
     if ( not defined $paper ) {
         $self->{paper} = $paper;
+        $self->{current_scan_options}->remove_frontend_option('paper');
         $self->signal_emit( 'changed-paper', $paper );
         return;
     }
@@ -1703,6 +1704,10 @@ sub set_paper {
         $options->by_name(SANE_NAME_SCAN_BR_Y)->{val}
     );
 
+    # forget the previous option info calls, as these are only interesting
+    # *whilst* setting a profile, and now we are starting from scratch
+    delete $self->{option_info};
+
     if ( not $paper_profile->num_backend_options ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
@@ -2474,7 +2479,6 @@ sub _set_option_profile {
             }
         );
         $self->set_option( $opt, $val, $profile->{uuid} );
-        $self->update_widget_value( $opt, $val );
     }
     else {
 
@@ -2510,7 +2514,11 @@ sub update_widget_value {
               . ( $opt->{type} == SANE_TYPE_BUTTON ? $EMPTY : " to '$val'." ) );
         $widget->signal_handler_block( $widget->{signal} );
         given ($widget) {
-            when ( $widget->isa('Gtk3::CheckButton') ) {
+            when (
+                     $widget->isa('Gtk3::CheckButton')
+                  or $widget->isa('Gtk3::Switch')
+              )
+            {
                 if ( $val eq $EMPTY ) { $val = 0 }
                 if ( $widget->get_active != $val ) {
                     $widget->set_active($val);
